@@ -1,14 +1,16 @@
-#!/usr/bin/perl
+#!/usr/bin/perl -w
 
 package UNIVERSAL::isa;
 
 use strict;
-use warnings;
+use vars qw/$VERSION $recursing/;
+
 use UNIVERSAL ();
 
 use Scalar::Util qw/blessed/;
+use warnings::register;
 
-our $VERSION = "0.02";
+$VERSION = "0.03";
 
 my $orig;
 BEGIN { $orig = \&UNIVERSAL::isa };
@@ -21,6 +23,7 @@ sub import {
 }
 
 sub UNIVERSAL::isa {
+	goto &$orig unless defined $_[0];
 
 	# not an object or a class name, we can skip
 	unless ( blessed($_[0]) )
@@ -47,13 +50,19 @@ sub UNIVERSAL::isa {
 
 	# if we've been called from an overridden isa, we are either SUPER:: or explicitly called.
 	# in both cases the original ISA behavior is expected.
-	our $recursing;
 	goto &$orig if $recursing;
 
 	# the last possible case is that 'isa' is overridden
 	local $recursing = 1;
 	my $obj = shift;
-	return $obj->isa(@_);	
+
+	if (warnings::enabled()) {
+		my $calling_sub  = ( caller( 1 ) )[3] || '';
+		warnings::warn( "Called UNIVERSAL::isa() as a function, not a method" )
+			if $calling_sub !~ /::isa$/;
+	}
+
+	return $obj->isa(@_);
 }
 
 __PACKAGE__;
@@ -84,6 +93,20 @@ called on those objects as a method.
 
 In all other cases the real C<UNIVERSAL::isa> is just called directly.
 
+=head1 WARNINGS
+
+If the lexical warnings pragma is available, a warning will be emitted for each
+naughty invocation of C<UNIVERSAL::isa>. These warnings can be silenced by
+saying:
+
+	no warnings 'UNIVERSAL::isa';
+
+in the lexical scope of the naughty code.
+
+=head1 SEE ALSO
+
+L<UNIVERSAL::can> for a more mature discussion of the problem at hand.
+
 =head1 AUTHORS
 
 Autrijus Tang <autrijus@autrijus.org>
